@@ -34,6 +34,8 @@
 #endif
 #include <gdk-card-image.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 /* An image file used in building the cards:
  * each file can contain multiple images and/or half images */
@@ -205,10 +207,10 @@ GtkTypeInfo gdk_card_deck_info =
   (GtkClassInitFunc) NULL
 };
 
-guint
+GtkType
 gdk_card_deck_get_type ()
 {
-  static guint type = 0;
+  static GtkType type = 0;
 
   if (!type)
     type = gtk_type_unique (gtk_object_get_type (), &gdk_card_deck_info);
@@ -223,6 +225,7 @@ is_image (const struct dirent* dent)
 #endif
 {
 	const char *type = gnome_vfs_mime_type_from_name (dent->d_name);
+
 	if (type == NULL ||
 	    strncmp (type, "image/", strlen ("image/")) != 0)
 		return FALSE;
