@@ -21,11 +21,20 @@
 #include <config.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <gconf/gconf-client.h>
 
 #include "io-gtk.h"
 #include "option.h"
 #include "score.h"
 
+GConfClient *freecell_gconf_client = NULL;
+
+void key_changed_callback (GConfClient *tmp_client, guint cnx_id,
+                           GConfEntry *tmp_entry, gpointer tmp_data)
+{
+  option_init ();
+}
+
 /* The entry of this program.  */
 int
 main (int argc, char **argv)
@@ -36,12 +45,25 @@ main (int argc, char **argv)
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_init("freecell", VERSION, argc, argv);
+  gnome_program_init ("freecell", VERSION,
+		  LIBGNOMEUI_MODULE,
+		  argc, argv,
+		  NULL);
+
+  freecell_gconf_client = gconf_client_get_default ();
+  gconf_client_add_dir (freecell_gconf_client, "/apps/freecell/option",
+    GCONF_CLIENT_PRELOAD_NONE, NULL);
+  gconf_client_notify_add (freecell_gconf_client, "/apps/freecell/option",
+    key_changed_callback, NULL, NULL, NULL);
+
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-cardgame.png");
   option_init();
   score_init();
 
   io_gtk_init ();
   io_gtk_loop ();
+
+  g_object_unref (G_OBJECT (freecell_gconf_client));
+
   return 0;
 }
