@@ -75,7 +75,7 @@ static GdkCursor *up_cursor;
 static gint stalled = 0;
 
 
-static void callback_new_really_callback (GtkWidget *widget, gpointer data);
+static void callback_new_really_callback ();
 
 static void to_destination_auto(void);
 
@@ -88,11 +88,11 @@ static void refresh_destination (gint index);
 static void refresh_all (void);
 
 
-static void callback_restart_lose (GtkWidget *widget, gpointer data);
+static void callback_restart_lose ();
 static void callback_restart_really (void);
-static void callback_new_with_lose (GtkWidget *widget, gpointer data);
+static void callback_new_with_lose ();
 static void callback_new_really (void);
-static void callback_exit_with_lose (GtkWidget *widget, gpointer data);
+static void callback_exit_with_lose ();
 static void callback_exit_really (void);
 
 static void inform_invalid_move (void);
@@ -117,11 +117,10 @@ io_gtk_init (void)
 
   
   /* make main window.  */
-  gtk_widget_push_visual (gdk_rgb_get_visual ());
-  gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+  gtk_widget_push_colormap (gdk_rgb_get_colormap ());
   main_window = gnome_app_new ("freecell", _("Freecell"));
-  gtk_signal_connect (GTK_OBJECT(main_window), "delete_event",
-		      GTK_SIGNAL_FUNC(callback_exit), NULL);
+  g_signal_connect (G_OBJECT(main_window), "delete_event",
+		      G_CALLBACK(callback_exit), NULL);
   gtk_widget_realize(main_window);
 
   card_draw_init(main_window);
@@ -136,13 +135,13 @@ io_gtk_init (void)
   for (i = 0; i < 4; i++)
     {
       freecell_drawing_areas[i] = gtk_drawing_area_new();
-      gtk_signal_connect (GTK_OBJECT(freecell_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(freecell_drawing_areas[i]),
 			  "expose_event",
-			  GTK_SIGNAL_FUNC (callback_freecell_expose),
+			  G_CALLBACK (callback_freecell_expose),
 			  GINT_TO_POINTER (i));
-      gtk_signal_connect (GTK_OBJECT(freecell_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(freecell_drawing_areas[i]),
 			  "button_press_event",
-			  GTK_SIGNAL_FUNC (callback_freecell_press),
+			  G_CALLBACK (callback_freecell_press),
 			  GINT_TO_POINTER (i));
       
       gtk_drawing_area_size (GTK_DRAWING_AREA(freecell_drawing_areas[i]),
@@ -159,13 +158,13 @@ io_gtk_init (void)
   for (i = 0; i < 4; i++)
     {
       destination_drawing_areas[i] = gtk_drawing_area_new();
-      gtk_signal_connect (GTK_OBJECT(destination_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(destination_drawing_areas[i]),
 			  "expose_event",
-			  GTK_SIGNAL_FUNC (callback_destination_expose),
+			  G_CALLBACK (callback_destination_expose),
 			  GINT_TO_POINTER (i));
-      gtk_signal_connect (GTK_OBJECT(destination_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(destination_drawing_areas[i]),
 			  "button_press_event",
-			  GTK_SIGNAL_FUNC (callback_destination_press),
+			  G_CALLBACK (callback_destination_press),
 			  GINT_TO_POINTER (i));
       gtk_drawing_area_size (GTK_DRAWING_AREA(destination_drawing_areas[i]),
 			     card_image_width(), card_image_height());
@@ -188,17 +187,17 @@ io_gtk_init (void)
   for (i = 0; i < 8; i++)
     {
       field_drawing_areas[i] = gtk_drawing_area_new();
-      gtk_signal_connect (GTK_OBJECT(field_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(field_drawing_areas[i]),
 			  "expose_event",
-			  GTK_SIGNAL_FUNC (callback_field_expose),
+			  G_CALLBACK (callback_field_expose),
 			  GINT_TO_POINTER (i));
-      gtk_signal_connect (GTK_OBJECT(field_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(field_drawing_areas[i]),
 			  "button_press_event",
-			  GTK_SIGNAL_FUNC (callback_field_press),
+			  G_CALLBACK (callback_field_press),
 			  GINT_TO_POINTER (i));
-      gtk_signal_connect (GTK_OBJECT(field_drawing_areas[i]),
+      g_signal_connect (G_OBJECT(field_drawing_areas[i]),
 			  "button_release_event",
-			  GTK_SIGNAL_FUNC (callback_field_release),
+			  G_CALLBACK (callback_field_release),
 			  GINT_TO_POINTER (i));
       gtk_drawing_area_size (GTK_DRAWING_AREA(field_drawing_areas[i]),
 			     FIELD_WIDTH,
@@ -222,7 +221,6 @@ io_gtk_init (void)
   up_cursor = gdk_cursor_new (GDK_SB_UP_ARROW);
 
   gtk_widget_pop_colormap ();
-  gtk_widget_pop_visual ();
 
   stalled = 1;
 }
@@ -319,31 +317,26 @@ callback_dialog_destroyed (GtkWidget *widget, gpointer data)
 void
 callback_restart (GtkWidget *widget, GdkEvent *event)
 {
-  if (mb)
-	  return;
-
   if (!stalled
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_message_box_new (_("Exit this game?"),
-				  GNOME_MESSAGE_BOX_QUESTION,
-				  GNOME_STOCK_BUTTON_YES,
-				  GNOME_STOCK_BUTTON_NO,
-				  NULL);
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-/*    gtk_window_set_modal(GTK_WINDOW (mb), TRUE); */
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-				"clicked",
-				GTK_SIGNAL_FUNC(callback_restart_lose),
-				NULL);
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-                                "destroy",
-				GTK_SIGNAL_FUNC(callback_dialog_destroyed),
-                                NULL);
+      int response = 0;
 
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_widget_show (mb);
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_QUESTION,
+		      GTK_BUTTONS_YES_NO,
+		      _("Exit this game?"),
+		      NULL);
+
+      response = gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy (mb);
+
+      if (response == GTK_RESPONSE_YES)
+      {
+	      callback_restart_lose();
+      }
     }
   else
     callback_restart_really();
@@ -351,15 +344,12 @@ callback_restart (GtkWidget *widget, GdkEvent *event)
 
 
 static void
-callback_restart_lose (GtkWidget *widget, gpointer data)
+callback_restart_lose ()
 {
-  if (GPOINTER_TO_INT (data) == 0)
-    {
-      score_add_lose ();
-      callback_restart_really ();
-    }
+	score_add_lose ();
+	callback_restart_really ();
 
-  mb = NULL;
+	mb = NULL;
 }
 
 static void
@@ -387,13 +377,10 @@ callback_cancel (GtkWidget *widget, gpointer data)
 }
 
 static void
-callback_seed_input (GtkWidget *widget, gpointer data)
+callback_seed_input (int seed)
 {
-  int seed;
   char buffer[64];
 
-  seed = atoi (gtk_entry_get_text (GTK_ENTRY (data)));
-  
   if (freecellgame)
     freecellgame_delete (freecellgame);
 
@@ -415,80 +402,79 @@ static void
 callback_new_with_seed_really (void)
 {
   GtkWidget *dialog, *entry, *label;
+  int response;
 
-  dialog = gnome_dialog_new (_("Seed"),
-			     GNOME_STOCK_BUTTON_OK,
-			     GNOME_STOCK_BUTTON_CANCEL, NULL);
-
-  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (main_window));
+  dialog = gtk_dialog_new_with_buttons (_("Seed"),
+		  GTK_WINDOW (main_window),
+		  0,
+		  GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+		  GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+		  NULL);
 
   label = gtk_label_new (_("Seed value:"));
-  gtk_box_pack_start_defaults (GTK_BOX(GNOME_DIALOG(dialog)->vbox), label);
+  gtk_box_pack_start_defaults (GTK_BOX (GTK_CHECK_CAST((dialog),
+				  GTK_TYPE_DIALOG, GtkDialog)->vbox), label);
   gtk_widget_show (label);
   
   entry = gtk_entry_new ();
-  gtk_box_pack_start_defaults (GTK_BOX(GNOME_DIALOG(dialog)->vbox), entry);
+  gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(dialog)->vbox), entry);
   gtk_widget_show (entry);
 
-  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 0,
-			       GTK_SIGNAL_FUNC (callback_seed_input), entry);
-  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 0,
-			       GTK_SIGNAL_FUNC (callback_cancel), dialog);
-  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 1,
-			       GTK_SIGNAL_FUNC (callback_cancel), dialog);
-  gtk_widget_show (dialog);
+  response = gtk_dialog_run (GTK_DIALOG(dialog));
+  gtk_widget_hide (dialog);
+
+  if (response == GTK_RESPONSE_ACCEPT)
+  {
+	  int seed;
+
+	  seed = atoi (gtk_entry_get_text (GTK_ENTRY (entry)));
+	  callback_seed_input (seed);
+  }
+
+  gtk_widget_destroy (dialog);
 }
 
 
 static void
-callback_new_with_seed_with_lose (GtkWidget *widget, gpointer data)
+callback_new_with_seed_with_lose ()
 {
-  if ( GPOINTER_TO_INT (data) == 0)
-    {
-      score_add_lose();
-      callback_new_with_seed_really();
-    }
-  mb = NULL;
+	score_add_lose();
+	callback_new_with_seed_really();
+
+	mb = NULL;
 }
 
 
 void
 callback_new_with_seed (GtkWidget *widget, GdkEvent *event)
 {
-  if (mb)
-	  return;
-
   if (!stalled
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_message_box_new (_("Exit this game?"),
-				  GNOME_MESSAGE_BOX_QUESTION,
-				  GNOME_STOCK_BUTTON_YES,
-				  GNOME_STOCK_BUTTON_NO,
-				  NULL);
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-/*    gtk_window_set_modal(GTK_WINDOW(mb), TRUE); */
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-				"clicked",
-				GTK_SIGNAL_FUNC(callback_new_with_seed_with_lose),
-				NULL);
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-                                "destroy",
-				GTK_SIGNAL_FUNC(callback_dialog_destroyed),
-                                NULL);
+      int response;
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_QUESTION,
+		      GTK_BUTTONS_YES_NO,
+		      _("Exit this game?"),
+		      NULL);
 
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_widget_show (mb);
+      response = gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy(mb);
+
+      if (response == GTK_RESPONSE_YES)
+      {
+	      callback_new_with_seed_with_lose();
+      }
     }
   else
     callback_new_with_seed_really();
 }
 
 static void
-callback_new_really_callback (GtkWidget *widget, gpointer data)
+callback_new_really_callback ()
 {
-  if (GPOINTER_TO_INT (data) == 0)
     callback_new_really ();
 }
 
@@ -514,45 +500,38 @@ callback_new_really (void)
 
 
 static void
-callback_new_with_lose (GtkWidget *widget, gpointer data)
+callback_new_with_lose ()
 {
-  if (GPOINTER_TO_INT (data) == 0)
-    {
-      score_add_lose();
-      callback_new_really();
-    }
-  mb = NULL;
+	score_add_lose();
+	callback_new_really();
+
+	mb = NULL;
 }
 
 
 void
 callback_new (GtkWidget *widget, GdkEvent *event)
 {
-  if (mb)
-	  return;
-
   if (!stalled
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_message_box_new (_("Exit this game?"),
-				  GNOME_MESSAGE_BOX_QUESTION,
-				  GNOME_STOCK_BUTTON_YES,
-				  GNOME_STOCK_BUTTON_NO,
-				  NULL);
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-/*    gtk_window_set_modal(GTK_WINDOW(mb), TRUE); */
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-				"clicked",
-				GTK_SIGNAL_FUNC(callback_new_with_lose),
-				NULL);
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-                                "destroy",
-				GTK_SIGNAL_FUNC(callback_dialog_destroyed),
-                                NULL);
+      int response;
 
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_widget_show (mb);
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_QUESTION,
+		      GTK_BUTTONS_YES_NO,
+		      _("Exit this game?"),
+		      NULL);
+
+      response = gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy(mb);
+
+      if (response == GTK_RESPONSE_YES)
+      {
+	      callback_new_with_lose();
+      }
     }
   else
     callback_new_really();
@@ -584,9 +563,7 @@ callback_option (GtkWidget *widget, GdkEvent *event)
 {
   GtkWidget *dialog;
 
-  dialog = option_dialog ();
-
-  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (main_window));
+  dialog = option_dialog (main_window);
 
   gtk_widget_show (dialog);
 }
@@ -597,14 +574,15 @@ callback_rule (GtkWidget *widget, GdkEvent *event)
 {
   GtkWidget *mb;
 
-  mb = gnome_message_box_new (_("Sorry, this feature is not (yet) implemented."),
-			      GNOME_MESSAGE_BOX_INFO,
-			      GNOME_STOCK_BUTTON_OK,
-			      NULL);
-  /*GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;*/
-  gtk_window_set_modal (GTK_WINDOW(mb), TRUE);
-  gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-  gtk_widget_show (mb);
+  mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		  GTK_DIALOG_MODAL,
+		  GTK_MESSAGE_INFO,
+		  GTK_BUTTONS_OK,
+		  _("Sorry, this feature is not (yet) implemented."),
+		  NULL);
+
+  gtk_dialog_run (GTK_DIALOG (mb));
+  gtk_widget_destroy (mb);
 }
 
 
@@ -613,48 +591,41 @@ callback_exit_really (void)
 {
   score_write();
   option_write();
-  gtk_main_quit();
+  exit (0);
 }
 
 static void
-callback_exit_with_lose (GtkWidget *widget, gpointer data)
+callback_exit_with_lose ()
 {
-  if (GPOINTER_TO_INT (data) == 0)
-    {
-      score_add_lose();
-      callback_exit_really();
-    }
-  mb = NULL;
+	score_add_lose();
+	callback_exit_really();
+
+	mb = NULL;
 }
 
 void
 callback_exit (GtkWidget *widget, GdkEvent *event)
 {
-  if (mb)
-	  return;
-
   if (!stalled
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_message_box_new (_("Exit this game?"),
-				  GNOME_MESSAGE_BOX_QUESTION,
-				  GNOME_STOCK_BUTTON_YES,
-				  GNOME_STOCK_BUTTON_NO,
-				  NULL);
+      int response;
 
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-/*    gtk_window_set_modal(GTK_WINDOW(mb), TRUE); */
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-				"clicked",
-				GTK_SIGNAL_FUNC(callback_exit_with_lose),
-				NULL);
-      gtk_signal_connect_object(GTK_OBJECT(mb),
-				"destroy",
-				GTK_SIGNAL_FUNC(callback_dialog_destroyed),
-				NULL);
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_widget_show (mb);
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_QUESTION,
+		      GTK_BUTTONS_YES_NO,
+		      _("Exit this game?"),
+		      NULL);
+
+      response = gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy (mb);
+
+      if (response == GTK_RESPONSE_YES)
+      {
+	      callback_exit_with_lose();
+      }
     }
   else
     callback_exit_really();
@@ -663,13 +634,13 @@ callback_exit (GtkWidget *widget, GdkEvent *event)
 void
 callback_about (GtkWidget *widget, GdkEvent *event)
 {
+  GdkPixbuf *pixbuf = NULL;
   GtkWidget *about;
   gchar *authors[] = {
     N_("Changwoo Ryu."),
     NULL
   };
 
-#ifdef ENABLE_NLS
 #ifndef ELEMENTS  
 #define ELEMENTS(x) (sizeof(x) / sizeof(x[0]))
 #endif /* ELEMENTS */
@@ -678,16 +649,25 @@ callback_about (GtkWidget *widget, GdkEvent *event)
 
     for (i = 0; i < (ELEMENTS(authors) - 1); i++)
       authors[i] = _(authors[i]);
-  }  
-#endif /* ENABLE_NLS */
-  
+  }
+
+  {
+	  char *filename = NULL;
+	  
+	  filename = gnome_program_locate_file (NULL,
+			  GNOME_FILE_DOMAIN_PIXMAP,
+			  "gnome-cardgame.png", FALSE, NULL);
+
+	  pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+  }
+
   about = gnome_about_new (_("Freecell"), VERSION,
 			   "(C) 1998 Free Software Foundation, Inc.",
 			   _("Reimplement the popular solitaire card game."),
 			   (const char **)authors,
 			   NULL,
 			   NULL,
-			   NULL);
+			   pixbuf);
   gtk_widget_show (about);
 }
 
@@ -696,14 +676,15 @@ inform_invalid_move (void)
 {
   GtkWidget *mb;
 
-  mb = gnome_message_box_new (_("That move is invalid."),
-			      GNOME_MESSAGE_BOX_ERROR,
-			      GNOME_STOCK_BUTTON_OK,
-			      NULL);
-  /*GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;*/
-  gtk_window_set_modal(GTK_WINDOW(mb), TRUE);
-  gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
+  mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		  GTK_DIALOG_MODAL,
+		  GTK_MESSAGE_ERROR,
+		  GTK_BUTTONS_OK,
+		  _("That move is invalid."),
+		  NULL);
+
   gtk_widget_show (mb);
+  gtk_dialog_run (GTK_DIALOG(mb));
 }
 
 static void
@@ -1186,36 +1167,40 @@ to_destination_auto(void)
     {
       freecellgame_delete_history(freecellgame);
 
-      mb = gnome_message_box_new (_("Sorry, there are no more valid moves."),
-				  GNOME_MESSAGE_BOX_INFO,
-				  GNOME_STOCK_BUTTON_OK,
-				  NULL);
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_INFO,
+		      GTK_BUTTONS_OK,
+		      _("Sorry, there are no more valid moves."),
+		      NULL);
+
+
+      gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy(mb);
 
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-      gtk_window_set_modal(GTK_WINDOW(mb), TRUE);
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_widget_show (mb);
       score_add_lose();
       stalled = 1;
     }
   
   if (freecellgame_is_finished(freecellgame))
     {
+      int response = 0;
+
       freecellgame_delete_history(freecellgame);
 
-      mb = gnome_message_box_new (_("Congratulations.  You won.\nDo you want to play again?"),
-				  GNOME_MESSAGE_BOX_QUESTION,
-				  GNOME_STOCK_BUTTON_YES,
-				  GNOME_STOCK_BUTTON_NO,
-				  NULL);
-/*    GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; */
-      gtk_window_set_modal(GTK_WINDOW(mb), TRUE);
-      gnome_dialog_set_parent (GNOME_DIALOG (mb), GTK_WINDOW (main_window));
-      gtk_signal_connect_object (GTK_OBJECT (mb),
-				 "clicked",
-				 GTK_SIGNAL_FUNC(callback_new_really_callback),
-				 NULL);
-      gtk_widget_show (mb);
+      mb = gtk_message_dialog_new (GTK_WINDOW (main_window),
+		      GTK_DIALOG_MODAL,
+		      GTK_MESSAGE_QUESTION,
+		      GTK_BUTTONS_YES_NO,
+		      _("Congratulations.  You won.\nDo you want to play again?"),
+		      NULL);
+
+      gtk_dialog_run (GTK_DIALOG(mb));
+      gtk_widget_destroy (mb);
+
+      if (response == GTK_RESPONSE_YES)
+	      callback_new_really_callback();
+      
       score_add_win();
       stalled = 1;
     }
