@@ -1,5 +1,4 @@
-/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*-
- *
+/*
  * gnect prefs.h
  *
  */
@@ -52,6 +51,8 @@ struct _Prefs {
 };
 
 
+void prefs_init(gint argc, gchar **argv);
+void prefs_free(void);
 void prefs_get(void);
 void prefs_save(void);
 void prefs_dialog(void);
