@@ -798,33 +798,28 @@ void quit_game_cb(GtkWidget *widget, gpointer data){
   gtk_main_quit();
 }
 
-static char *nstr(int n){
-  char buf[20]; sprintf(buf, "%d", n);
-  return g_strdup(buf);
-}
-
 static int save_state(GnomeClient *client,gint phase, 
 		      GnomeRestartStyle save_style, gint shutdown,
 		      GnomeInteractStyle interact_style, gint fast,
 		      gpointer client_data){
   char *argv[20];
   int i;
-  gint xpos, ypos;
+  int xpos, ypos;
   
   gdk_window_get_origin(window->window, &xpos, &ypos);
   
   i = 0;
   argv[i++] = (char *)client_data;
   argv[i++] = "-x";
-  argv[i++] = nstr(xpos);
+  argv[i++] = g_strdup_printf("%d",xpos);
   argv[i++] = "-y";
-  argv[i++] = nstr(ypos);
+  argv[i++] = g_strdup_printf("%d",ypos);
 
   gnome_client_set_restart_command(client, i, argv);
   gnome_client_set_clone_command(client, 0, NULL);
   
-  free(argv[2]);
-  free(argv[4]);
+  g_free(argv[2]);
+  g_free(argv[4]);
   
   return TRUE;
 }
