@@ -17,12 +17,10 @@
 #include <dirent.h>
 
 #include <config.h>
-#define GNOME_DISABLE_DEPRECATED
-#define GTK_DISABLE_DEPRECATED
-#define G_DISABLE_DEPRECATED
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gconf/gconf-client.h>
 
 
 #define STONE_SIZE 40
@@ -46,6 +44,8 @@ static gchar *scenario;
 static gint restarted;
 static gboolean game_over = FALSE;
 
+/* Prefs */
+GConfClient *conf_client = NULL;
 char *selected_scenario = NULL;
 
 static struct ball {
@@ -58,9 +58,6 @@ static int nstones;
 static int ncolors;
 static int sync_stones = 0;
 
-/* This is used only while parsing arguments.  */
-static gchar *fname;
-
 #define mapx(x) (x)
 #define mapy(y) (STONE_LINES-1-(y))
 
@@ -181,9 +178,9 @@ mark_balls (int x, int y)
 	tagged_count = flood_fill (x, y, field [x][y].color);
 	
 	if (tagged_count > 1) {
-		char b[3];
+		char *b;
 		ball_timeout_id = gtk_timeout_add (100, move_tagged_balls, 0);
-		sprintf (b, "%.2d", tagged_count);
+		b = g_strdup_printf ("%.2d", tagged_count);
 		gtk_label_set_text (GTK_LABEL(markedw), b);
 	} else
 		gtk_label_set_text (GTK_LABEL(markedw), "00");
@@ -250,10 +247,10 @@ compress_x ()
 static void
 set_score (int new_score)
 {
-	char b [20];
+	char *b = NULL;
 	
 	score = new_score;
-	sprintf (b, "%.5d", score);
+	b = g_strdup_printf ("%.5d", score);
 	gtk_label_set_text (GTK_LABEL(scorew), b);
 }
 
@@ -476,11 +473,10 @@ game_new_callback (GtkWidget *widget, void *data)
 }
 
 static void
-set_selection (GtkWidget *widget, void *data)
+set_selection (GtkWidget *widget, gpointer data)
 {
 	load_scenario (data);
-	gnome_config_set_string ("/same-gnome/Preferences/Scenario", data);
-	gnome_config_sync();
+	gconf_client_set_string (conf_client, "/apps/same-gnome/tileset", data, NULL);
 }
 
 static void
@@ -524,13 +520,13 @@ pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 	if (response == GTK_RESPONSE_OK)
 	{
 		g_free (selected_scenario);
-		selected_scenario = gnome_config_get_string ("/same-gnome/Preferences/Scenario");
+		selected_scenario = gconf_client_get_string (conf_client, "/apps/same-gnome/tileset", NULL);
 	} else {
 		/* Restore previous scenario */
 		load_scenario (selected_scenario);
-		gnome_config_set_string ("/same-gnome/Preferences/Scenario", 
-			selected_scenario);
-		gnome_config_sync();
+
+		gconf_client_set_string (conf_client, "/apps/same-gnome/tileset", 
+			selected_scenario, NULL);
 	}
 
 	gtk_widget_destroy (pref_dialog);
@@ -592,6 +588,7 @@ game_preferences_callback (GtkWidget *widget, void *data)
 static int
 game_about_callback (GtkWidget *widget, void *data)
 {
+	GdkPixbuf *pixbuf = NULL;
 	static GtkWidget *about = NULL;
 	const gchar *authors[] = {
 		"Miguel de Icaza.",
@@ -609,6 +606,19 @@ game_about_callback (GtkWidget *widget, void *data)
 		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
+
+	{
+		char *filename = NULL;
+
+		filename = gnome_program_locate_file (NULL,
+				GNOME_FILE_DOMAIN_PIXMAP,  ("gnome-gsame.png"),
+				TRUE, NULL);
+		if (filename != NULL)
+		{
+			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
+			g_free (filename);
+		}
+	}
 	
 	about = gnome_about_new (_("The Same Gnome"), VERSION,
 				 "(C) 1997-1998 the Free Software Foundation",
@@ -616,7 +626,7 @@ game_about_callback (GtkWidget *widget, void *data)
 				 (const char **)authors,
 				 (const char **)documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-                                 NULL);
+                                 pixbuf);
 	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
 	gtk_widget_show (about);
 
@@ -695,31 +705,28 @@ save_state (GnomeClient *client,
 	    gint fast,
 	    gpointer client_data)
 {
-	const gchar *prefix = gnome_client_get_config_prefix (client);
 	gchar *argv []= { "rm", "-r", NULL };
 	gchar *buf;
 	struct ball *f = (struct ball*) field;
 	int i;  
 	
-	gnome_config_push_prefix (prefix);
-	
-	gnome_config_set_int ("Game/Score", score);
-	gnome_config_set_int ("Game/NStones", sync_stones ? 1 : nstones);
+	gconf_client_set_int (conf_client, "/apps/same-gnome/score", score, NULL);
+	gconf_client_set_int (conf_client, "/apps/same-gnome/nstones", sync_stones ? 1 : nstones, NULL);
 	
 	buf= g_malloc (STONE_COLS*STONE_LINES+1);
-	
+
 	for (i = 0 ; i < (STONE_COLS*STONE_LINES); i++){
 		buf [i]= f [i].color + 'a';
 	}
 	buf [STONE_COLS*STONE_LINES]= '\0';
-	gnome_config_set_string ("Game/Field", buf);
+	gconf_client_set_string (conf_client, "/apps/same-gnome/field", buf, NULL);
 	g_free(buf);
-	
-	gnome_config_pop_prefix ();
-	gnome_config_sync();
-	
+
+	/* FIXME if anybody knows what this does...
 	argv[2]= gnome_config_get_real_path (prefix);
 	gnome_client_set_discard_command (client, 3, argv);
+	gnome_client_set_discard_command (client, 2, argv);
+	*/
 	
 	return TRUE;
 }
@@ -732,10 +739,10 @@ restart (void)
 	struct ball *f = (struct ball*) field;
 	int i;
 	
-	score = gnome_config_get_int_with_default ("Game/Score", 0);
-	nstones = gnome_config_get_int_with_default ("Game/NStones", 0);
+	score = gconf_client_get_int (conf_client, "/apps/same-gnome/score", NULL);
+	nstones = gconf_client_get_int (conf_client, "/apps/same-gnome/nstones", NULL);
 	
-	buf = gnome_config_get_string_with_default ("Game/Field", NULL);  
+	buf = gconf_client_get_string (conf_client, "/apps/same-gnome/field", NULL);
 
 	if (buf) {
 		for (i= 0; i < (STONE_COLS*STONE_LINES); i++) 
@@ -756,11 +763,6 @@ client_die (GnomeClient *client, gpointer client_data)
 	return FALSE;
 }
 
-static const struct poptOption options[] = {
-	{ "scenario", 's', POPT_ARG_STRING, &fname, 0, N_("Set game scenario"), N_("NAME") },
-	{ NULL, '\0', 0, NULL, 0 }
-};
-
 #ifndef GNOME_CLIENT_RESTARTED
 #define GNOME_CLIENT_RESTARTED(client) \
 (GNOME_CLIENT_CONNECTED (client) && \
@@ -772,6 +774,11 @@ static const struct poptOption options[] = {
 int
 main (int argc, char *argv [])
 {
+	static char *fname;
+	static const struct poptOption options[] = {
+		{ "scenario", 's', POPT_ARG_STRING, &fname, 0, N_("Set game scenario"), N_("NAME") },
+		{ NULL, '\0', 0, NULL, 0 }
+	};
 	GtkWidget *label, *separator;
 	GnomeClient *client;
 
@@ -792,14 +799,13 @@ main (int argc, char *argv [])
 			  G_CALLBACK (client_die), NULL);
 
 	if (GNOME_CLIENT_RESTARTED (client)){
-		gnome_config_push_prefix (gnome_client_get_config_prefix (client));
-	    
 		restart ();
 		restarted = 1;
-		
-		gnome_config_pop_prefix ();
 	}
 
+	/* Get the default GConfClient */
+	conf_client = gconf_client_get_default ();
+
 	srand (time (NULL));
 
 	app = gnome_app_new("same-gnome", _("Same Gnome"));
@@ -822,8 +828,8 @@ main (int argc, char *argv [])
 	gnome_app_set_contents (GNOME_APP (app), vb);
 
 	if (!fname) {
-		fname = gnome_config_get_string
-			("/same-gnome/Preferences/Scenario=stones.png");
+		fname = gconf_client_get_string
+			(conf_client, "/apps/same-gnome/tileset", NULL);
 	}
 
 	create_same_board (fname);
